@@ -1,8 +1,8 @@
+"""Module provider for auto"""
 from __future__ import absolute_import
 import argparse
 import importlib
 import logging
-import os
 import pkgutil
 import re
 import subprocess
@@ -23,14 +23,13 @@ LOGGER = logging.getLogger(__name__)
 def _get_available_providers():
     available_providers = {}
     for _, modname, _ in pkgutil.iter_modules(providers.__path__):
-        if modname != 'base' and modname != 'auto':
+        if modname not in ('base', 'auto'):
             try:
                 available_providers[modname] = importlib.import_module(
                     'lexicon.providers.' + modname)
             except ImportError:
-                LOGGER.warn('Warning, the provider {0} cannot be loaded due '
-                            'to missing optional dependencies.'
-                            .format(modname))
+                LOGGER.warning('Warning, the provider %s cannot be loaded due '
+                               'to missing optional dependencies.', modname)
 
     return available_providers
 
@@ -42,7 +41,7 @@ def _get_ns_records_domains_for_domain(domain):
     tlds = [tldextract.extract(ns_entry)
             for ns_entry in _get_ns_records_for_domain(domain)]
 
-    return set(['{0}.{1}'.format(tld.domain, tld.suffix) for tld in tlds])
+    return {'{0}.{1}'.format(tld.domain, tld.suffix) for tld in tlds}
 
 
 def _get_ns_records_for_domain(domain):
@@ -50,12 +49,13 @@ def _get_ns_records_for_domain(domain):
     try:
         output = subprocess.check_output(['nslookup', '-querytype=NS', domain],
                                          stderr=subprocess.STDOUT, universal_newlines=True)
-    except subprocess.CalledProcessError as e:
-        if 'NXDOMAIN' in e.output:
+    except subprocess.CalledProcessError as error:
+        if 'NXDOMAIN' in error.output:
             raise ValueError(
                 'Error, domain {0} could not be resolved.'.format(domain))
+        output = error.output
 
-    pattern = re.compile(r'nameserver = (.*?)\.*{0}'.format(os.linesep))
+    pattern = re.compile(r'nameserver = (.*?)\.*\n')
     match = pattern.findall(output)
 
     if not match:
@@ -73,8 +73,8 @@ def _relevant_provider_for_domain(domain):
         ns_domains = provider_module.NAMESERVER_DOMAINS
 
         # Test plain domain string comparison
-        if set([ns_domain for ns_domain in ns_domains
-                if isinstance(ns_domain, six.string_types)]) & nameserver_domains:
+        if {ns_domain for ns_domain in ns_domains
+                if isinstance(ns_domain, six.string_types)} & nameserver_domains:
             relevant_providers.append((provider_name, provider_module))
             continue
 
@@ -91,21 +91,22 @@ def _relevant_provider_for_domain(domain):
                          'Found nameservers domains are {1}'.format(domain, nameserver_domains))
 
     if len(relevant_providers) > 1:
-        LOGGER.warn('Warning, multiple DNS providers have been found for given domain {0}, '
-                    'first one will be used: {1} '
-                    'This may indicate a misconfiguration in one or more provider.'
-                    .format(domain, relevant_providers))
+        LOGGER.warning('Warning, multiple DNS providers have been found for given domain %s, '
+                       'first one will be used: %s\n'
+                       'This may indicate a misconfiguration in one or more provider.',
+                       domain, relevant_providers)
 
     return relevant_providers[0]
 
 
-def ProviderParser(subparser):
+def provider_parser(subparser):
+    """Configure provider parser for auto provider"""
     subparser.description = '''
         Provider 'auto' enables the Lexicon provider auto-discovery.
         Based on the nameservers declared for the given domain,
         Lexicon will try to find the DNS provider holding the DNS zone if it is supported.
         Actual DNS zone read/write operations will be delegated to the provider found:
-        every environment variable or command line specific to this provider 
+        every environment variable or command line specific to this provider
         can be passed to Lexicon and will be processed accordingly.
         '''
     subparser.add_argument("--mapping-override", metavar="[DOMAIN]:[PROVIDER], ...",
@@ -116,12 +117,12 @@ def ProviderParser(subparser):
     # Explore and load the arguments available for every provider into the 'auto' provider.
     for provider_name, provider_module in AVAILABLE_PROVIDERS.items():
         parser = argparse.ArgumentParser(add_help=False)
-        provider_module.ProviderParser(parser)
+        provider_module.provider_parser(parser)
 
-        for action in parser._actions:
+        for action in parser._actions:  # pylint: disable=protected-access
             action.option_strings = [re.sub(
                 r'^--(.*)$', r'--{0}-\1'.format(provider_name), option)
-                for option in action.option_strings]
+                                     for option in action.option_strings]
             action.dest = 'auto_{0}_{1}'.format(provider_name, action.dest)
             subparser._add_action(action)  # pylint: disable=protected-access
 
@@ -130,9 +131,7 @@ def ProviderParser(subparser):
 # but __getattr__ is called only if the parameter/method cannot be found in the
 # current Provider hierarchy. If it is object, it will be the case for every relevant
 # call in the Lexicon library.
-
-
-class Provider(object):
+class Provider(object):  # pylint: disable=useless-object-inheritance
     """
     Implementation of the provider 'auto'.
     For the given domain, it will resolve the actual Provider class to use by inspecting the
@@ -143,7 +142,6 @@ class Provider(object):
     the resolved provider if it respect the naming convention: --[provider]-[parameter_name] for a
     command line parameter, or LEXICON_[PROVIDER]_PARAMETER_NAME for a environment variable.
     """
-
     def __init__(self, config):
         if not isinstance(config, ConfigResolver):
             # If config is a plain dict, we are in a legacy situation.
@@ -156,14 +154,13 @@ class Provider(object):
         self.domain = config.resolve('lexicon:domain')
         self.proxy_provider = None
 
-    def authenticate(self):
+    def authenticate(self):  # pylint: disable=too-many-locals
         """
         Launch the authentication process: for 'auto' provider, it means first to find the relevant
-        provider, then call its authenticate() method. Almost every subsequent operation will then 
+        provider, then call its authenticate() method. Almost every subsequent operation will then
         be delegated to that provider.
         """
         mapping_override = self.config.resolve('lexicon:auto:mapping_override')
-        print(mapping_override)
         mapping_override_processed = {}
         if mapping_override:
             for one_mapping in mapping_override.split(','):
@@ -188,14 +185,14 @@ class Provider(object):
         new_config.with_dict({'lexicon:provider_name': provider_name})
 
         target_prefix = 'auto_{0}_'.format(provider_name)
-        for configSource in self.config._config_sources:
-            if not isinstance(configSource, ArgsConfigSource):
-                new_config.with_config_source(configSource)
+        for config_source in self.config._config_sources:  # pylint: disable=protected-access
+            if not isinstance(config_source, ArgsConfigSource):
+                new_config.with_config_source(config_source)
             else:
                 # ArgsConfigSource needs to be reprocessed to rescope the provided
                 # args to the delegate provider
                 new_dict = {}
-                for key, value in configSource._parameters.items():
+                for key, value in config_source._parameters.items():  # pylint: disable=protected-access
                     if key.startswith(target_prefix):
                         new_param_name = re.sub(
                             '^{0}'.format(target_prefix), '', key)
